//~~ dealer.hh ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
/*
 *  description
 *
 * @author:    Andy Pippin   <pippin@cs.ucsb.edu>
 * @version:   $Date: 2004/05/30 00:58:17 $   $Revision: 1.5 $
 * @requires:   
 */

//==[ Identity ]================================================================

#ifndef  __DEALER_HH__
#define  __DEALER_HH__

#ifdef __DEALER_CC__
static char* hdrIdentity =
    "$Id: dealer.hh,v 1.5 2004/05/30 00:58:17 pippin Exp $";
#endif


//==[ Includes ]================================================================

#include  <string>


//==[ Typedefs/Structures/Enums ]===============================================

//==[ Constants ]===============================================================

//==[ Variables ]===============================================================

//==[ Classes ]=================================================================

class Dealer
{
  public: //====================================================================

  //~ Member Typedefs/Structures/Enums ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  //~ Constants ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

  //~ Constructors ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    //----------------------------------------------------------------------
    /**
     *  Create an instance with no arguments.
     */
    Dealer(
        void
    );


    //----------------------------------------------------------------------
    /**
     *  Default destructor.
     */
    	    virtual
    ~Dealer(
        void
    );


  //~~ Getters ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    /**  Get the password (input) filename of the dealer.  */
    virtual char*  getPasswordFilename( void );

    /**  Get the results (output) filename of the dealer.  */
    virtual char*  getResultsFilename( void );


  //~~ Setters ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    /**  Set the password filename of the dealer.  */
    virtual void setPasswordFilename( char* passwordFilename );

    /**  Set the output filename of the dealer.  */
    virtual void  setResultsFilename( char* resultsFilename );


  //~~ Methods ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    //----------------------------------------------------------------------
    /**
     *  Begin execution using the default password file and options.
     *
     * @return   Exit code (0 -=> no errors).
     */
    virtual int  start(
	void
    );


    //----------------------------------------------------------------------
    /**
     *  Begin execution using the specified password file and options.
     *
     * @param   passwordFile  The file with the passwords (NULL --> default).
     * @param   resultsFile   The file for the results (NULL --> default).
     * @param   wordlistFile   The file for the wordlist (NULL --> default).
     * @return  Exit code (0 -=> no errors).
     */
    virtual int  start(
        char*  passwordFilename,
        char*  resultsFilename,
        char*  wordlistFilename
    );


    //----------------------------------------------------------------------
    /**
     *  Send all the workers a shutdown message.
     */
    virtual void  stop(
        void
    );


    //----------------------------------------------------------------------
    /**
     *  Make a string representation out of the instance.
     *
     * @param   prefix  Prepend this to each line (NULL --> "").
     * @param   label   Top-level label (NULL --> Dealer).
     * @return  XML representation.
     */
            virtual char*
    toString(
	char*  prefix =NULL,
	char*  label  =NULL
    );


  protected: //===============================================================

  private: //=================================================================

    //~~ Member variables ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    /**  Name of the password (input) filename to use.  */
    char*  passwordFilename;

    /**  Name of the results (output) filename to use.  */
    char*  resultsFilename;

    /**  Name of the wordlist filename to use.  */
    char*  wordlistFilename;


    //----------------------------------------------------------------------
    /**
     *  Initialize the member variables.
     */
    	    virtual void
    Initialize(
        void
    );

};


//==[ Prototypes ]==============================================================

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#endif // __DEALER_HH__
