/*
 * This file is part of John the Ripper password cracker,
 * Copyright (c) 1996-98 by Solar Designer
 */

#include <stdio.h>
#include <string.h>

#include "misc.h"
#include "params.h"
#include "memory.h"

static char    *home_path = NULL;
static size_t   home_path_length;

void 
path_init(char **argv)
{
	char           *pos;

	if (!argv[0])
		error();

	if (!home_path && (pos = strrchr(argv[0], '/')))
	{
		home_path_length = pos - argv[0] + 1;
		if (home_path_length >= PATH_BUFFER_SIZE)
			home_path_length = PATH_BUFFER_SIZE - 1;

		home_path = mem_alloc(PATH_BUFFER_SIZE);

		memcpy(home_path, argv[0], home_path_length);
	}
}

char           *
path_expand(char *name)
{

//--- Begin MPI modifications ------------------------------------------------//
//                                                                            //
//    Every John process needs an exclusive lock on the recover file.  With   //
//  the MPI version, recovery from an interruption is not a concern.  So the  //
//  recovery file has a ".%p" in it, which will get trapped and replaced      //
//  with the current process id.  - Andy Pippin                               //
//                                                                            //
        char   copyBuffer[ FILENAME_MAX ];
	char   pidBuffer[ 16 ];
	char*  percentP1;
	char*  percentP2;

	if( NULL != (percentP1 = strstr( name, "%p" )) )
	{
	    strncpy( copyBuffer, name, FILENAME_MAX );
	    percentP2 = strstr( copyBuffer, "%p" );
	    *percentP2 = '\0';
	    snprintf( pidBuffer, sizeof(pidBuffer), "%d", getpid() );
	    strncat( copyBuffer, pidBuffer, FILENAME_MAX );
	    strncat( copyBuffer, percentP1+2, FILENAME_MAX );
	    name = copyBuffer;
	}
//                                                                            //
//--- End MPI modifications --------------------------------------------------//


	if (!strncmp(name, "~/", 2))
	{
		if (home_path)
		{
			strnzcpy(&home_path[home_path_length], &name[2],
				 PATH_BUFFER_SIZE - home_path_length);

			return home_path;
		}
		else
			return name + 2;
	}
	else
		return name;
}

void 
path_done()
{
	mem_free((void **) &home_path);
}
