//~~ john-mpi.cc ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
/**
 *  Entry point into main routine.
 *
 * @author   Andy Pippin  <pippin@cs.ucsb.edu>
 * @version  $Revision: 1.10 $  $Date: 2004/06/03 14:19:39 $
 * @require  
 *
 */

//-[ Identity ]-----------------------------------------------------------------

#define __JOHN_MPI_CC__
static char* srcIdent =
    "$Id: john-mpi.cc,v 1.10 2004/06/03 14:19:39 pippin Exp $";


//-[ Includes ]-----------------------------------------------------------------

// This because JtR has it's own (old) getopt.h.
//#include  <getopt.h>
#include  </usr/include/getopt.h>

#include  <stdio.h>
#include  <stdlib.h>
#include  <string.h>
#include  "mpi.h"

#include  "john-mpi.hh"
#include  "workmsg.hh"
#include  "dealer.hh"
#include  "crack.hh"
#include  "usage.hh"


//-[ Typedefs/Structures/Enums ]------------------------------------------------

//-[ Constants ]----------------------------------------------------------------

static char const  OPTION_Input      = 'i';
static char const  OPTION_MaxPasswds = 'i';
static char const  OPTION_Output     = 'o';
static char const  OPTION_WordList   = 'w';
static char const  OPTION_DealerWait =  2;
//
static char const  OPTION_Help       = 'h';
static char const  OPTION_Quiet      = 'q';
static char const  OPTION_Verbose    = 'v';
static char const  OPTION_Usage      =  1;
//
static char* const  SHORT_OPTIONS = "i:m:o:w:hqv";
//

// Command line options.
//
static struct option  LONG_OPTIONS[] = 
{
    { "dealerwait",	no_argument,		NULL,	OPTION_DealerWait },
    { "input",		required_argument,	NULL,	OPTION_Input      },
    { "max",		required_argument,	NULL,	OPTION_MaxPasswds },
    { "output",		required_argument,	NULL,	OPTION_Output     },
    { "password",	required_argument,	NULL,	OPTION_Input      },
    { "passwords",	required_argument,	NULL,	OPTION_Input      },
    { "results",	required_argument,	NULL,	OPTION_Output     },
    { "wordlist",	required_argument,	NULL,	OPTION_WordList   },

    { "help",		optional_argument,	NULL,	OPTION_Help       },
    { "quiet",		no_argument,		NULL,	OPTION_Quiet      },
    { "usage",		no_argument,		NULL,	OPTION_Usage      },
    { "verbose",	optional_argument,	NULL,	OPTION_Verbose    },

    { NULL,		no_argument,		NULL,	0                 }
};


namespace john_mpi
{
    /**  Default verbose level. */
    int const    DEFAULT_verboseLevel = 1;

    /**  Default invocation name. */
    char* const  DEFAULT_cmdName = "john-mpi";

    /**  Default verbosity.  */
    int const    DEFAULT_Verbosity = 1;

    /**  Default maximum number of passwords per message.  */
    int const    DEFAULT_MaxPasswords = 20;

    /**  Default maximum number of passwords per message.  */
    char* const  DEFAULT_WordlistFilename = "wordlist";

    /**  Default maximum number of passwords per message.  */
    char* const  DEFAULT_PasswordFilename = "passwd";

    /**  Default maximum number of passwords per message.  */
    char* const  DEFAULT_ResultsFilename = "results";

    /**  Default password file (for workers).  */
    char* const  DEFAULT_CrackPasswd = "/tmp/john-passwd";

    /**  Time the dealer while waiting?  */
    bool const   DEFAULT_DealerWaitTime = true;
};


//-[ Variables ]----------------------------------------------------------------

namespace  john_mpi
{
    /**  Current verbose level. */
    int  verboseLevel = DEFAULT_verboseLevel;

    /**  Current invocation name. */
    char*  cmdName = DEFAULT_cmdName;

    /**  Current password file.  */
    char   passwdFile[ FILENAME_MAX ] = { '\0' };

    /**  Recording dealer wait time.  */
    bool   dealerWaitTime = DEFAULT_DealerWaitTime;

    /**  Total dealer wait time.  */
    struct timeval  totalDealerWaitTime = { '\0' };
};


//-[ Prototypes ]---------------------------------------------------------------

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
/**
 *  Main entry point into program.  (Like, duh!)
 *
 * @param   numArgs  Number of arguments on the command line.
 * @param   args     Arguments from the command line.
 * @return  Exit code.
 */
	int 
main(
    int		numArgs,
    char*	args[]
);


//-[ Implementation ]-----------------------------------------------------------

	int
main(
    int    numArgs,
    char*  cmdLine[]
){
    // Local variables.
    //
    int   exitCode = 0;				// Exit code.
    int   shortOption = 0;			// Short option value.
    int   longOption = 0;			// Long option value.
    char  cmdError = 0;				// Command line parsing error.
    char* cmdName;				// Command name.

    char*  passwordFilename = john_mpi::DEFAULT_PasswordFilename;
    char*  resultsFilename  = john_mpi::DEFAULT_ResultsFilename;
    char*  wordlistFilename = john_mpi::DEFAULT_WordlistFilename;


    // Initialize.
    //
    cmdName = cmdLine[ 0 ];

    // Parse the command line arguments.
    //
    while( EOF != (shortOption = getopt_long( numArgs, cmdLine, SHORT_OPTIONS,
			LONG_OPTIONS, &longOption )) )
    {
	switch( shortOption )
	{
	  // *** PASSWORD FILE ***
	  //
	  case  OPTION_Input:
	    passwordFilename = optarg;
            break;


	  // *** RESULTS FILE ***
	  //
	  case  OPTION_Output:
	    resultsFilename = optarg;
            break;


	  // *** WORDLIST FILE ***
	  //
	  case  OPTION_WordList:
	    wordlistFilename = optarg;
            break;


	  // *** DEALER WAIT TIME ***
	  //
	  case  OPTION_DealerWait:
	    john_mpi::dealerWaitTime = !john_mpi::DEFAULT_DealerWaitTime;
            break;


	  // *** HELP ***
	  //
	  case  OPTION_Help:
	    Usage::help( optarg );
            return( 0 );
	    // ***                                    ***
	    // *** DOES NOT CONTINUE -- PROGRAM EXITS ***
	    // ***                                    ***


          // *** USAGE ***
          //
          case  OPTION_Usage:
	    Usage::print();
            return( 0 );
	    // ***                                    ***
	    // *** DOES NOT CONTINUE -- PROGRAM EXITS ***
	    // ***                                    ***


          // *** VERBOSE ***
          //
          case  OPTION_Verbose:
            if( NULL == optarg )
                john_mpi::verboseLevel++;
            else
                john_mpi::verboseLevel = atoi( optarg );
            break;


          // *** QUIET ***
          //
          case  OPTION_Quiet:
	    john_mpi::verboseLevel = 0;
            break;


	  // *** ERROR ***
	  //
	  case  '?':
	  default:
	    cmdError = 1;
	    break;
	}
    }


    // Make sure we have the right number of items to work with.
    //
    if( 0 == (numArgs - optind) )
    {
//        cmdError = 1;
    }


    // Print error message.
    //
    if( cmdError )
    {
	printf( "Need to print some usage!\n" );
	exitCode = 1;
    }

    // Do the work.
    //
    else
    {
	MPI::Init( numArgs, cmdLine );

	if( 0 == MPI::COMM_WORLD.Get_rank() )
	{
	    Dealer* dealer = new Dealer();
	    exitCode = dealer->start( passwordFilename, resultsFilename,
					    wordlistFilename );
	}
	else
	{
	    Worker* worker = new Crack( wordlistFilename );
	    exitCode = worker->start();
	}

	MPI::Finalize();
    }


    // Exit.
    //
    return( exitCode );
}


//= End of file ================================================================
