//~~ trace.cc ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
/*
 *  Writes information to a log file.
 *
 * @author:    Andy Pippin   <pippin@cs.ucsb.edu>
 * @version:   $Date: 2004/05/17 03:07:43 $   $Revision: 1.2 $
 * @requires:   
 */


//==[ Identity ]================================================================

#define  __TRACE_CC__
static char* srcIdentity = 
    "$Id: trace.cc,v 1.2 2004/05/17 03:07:43 pippin Exp $";


//==[ Includes ]================================================================

#include  <stdarg.h>
#include  <stdio.h>
#include  <unistd.h>
#include  "mpi.h"

#include "trace.hh"


//==[ Typedefs/Structures/Enums ]===============================================

//==[ Constants ]===============================================================

//==[ Variables ]===============================================================

//==[ Classes ]=================================================================

//==[ Prototypes ]==============================================================

//==[ Implementation ]==========================================================


	bool
trace::dprintf(
    char*  file,
    int    line,
    char*  fmt,
    ...
){
#ifdef DEBUG
    char   prefix[ 32 ];

    // Try an get the rank number.
    //
    if( MPI::Is_initialized() )
        printf( "[%d (%d)] ", MPI::COMM_WORLD.Get_rank(), getpid() );
    else
        printf( "[(%d)] ", getpid() );

    // Display the appropriate prefix.
    //
    if( (NULL != file) && (0 < line) )
	printf( "%s:%d  ", file, line );
    else if( (NULL == file) && (0 < line) )
	printf( "%d  ", line );
    else if( (NULL != file) && (0 >= line) )
	printf( "%s  ", file );

    // Print the real text.
    //
    va_list  varArgs;
    va_start( varArgs, fmt );
    vprintf( fmt, varArgs );
    fflush( stdout );
    va_end( varArgs );


#endif // DEBUG
    return ( true );
}


//------------------------------------------------------------------------------

	bool
trace::log(
    char*  file,
    int    line,
    char*  fmt,
    ...
){
#ifdef DEBUG
    char   outputFilename[ FILENAME_MAX ];
    FILE*  output;
    char   prefix[ 32 ];

    // Try an get the rank number.
    //
    if( MPI::Is_initialized() )
        snprintf( outputFilename, sizeof(outputFilename), "tracelog.%02d",
	    MPI::COMM_WORLD.Get_rank() );
    else
        snprintf( outputFilename, sizeof(outputFilename), "tracelog" );

    // print to the trace log.
    //
    if( NULL != (output = fopen( outputFilename, "a" )) )
    {
	// Display the appropriate prefix.
	//
	if( (NULL != file) && (0 < line) )
	    fprintf( output, "%s:%d  ", file, line );
	else if( (NULL == file) && (0 < line) )
	    fprintf( output, "%d  ", line );
	else if( (NULL != file) && (0 >= line) )
	    fprintf( output, "%s  ", file );

	// Print the real text.
	//
        va_list  varArgs;
	va_start( varArgs, fmt );
	vfprintf( output, fmt, varArgs );
	va_end( varArgs );

	// Close up shop.
	//
	fflush( output );
	fclose( output );
    }

#endif // DEBUG
    return( true );
}


//------------------------------------------------------------------------------

	bool
trace::clear(
    void
){
#ifdef DEBUG
    char   outputFilename[ FILENAME_MAX ];
    FILE*  output;
    
    // Try an get the rank number.
    //
    if( MPI::Is_initialized() )
        snprintf( outputFilename, sizeof(outputFilename), "tracelog.%02d",
	    MPI::COMM_WORLD.Get_rank() );
    else
        snprintf( outputFilename, sizeof(outputFilename), "tracelog" );

    if( NULL != (output = fopen( outputFilename, "w" )) )
    {
        fclose( output );
    }

#endif // DEBUG
    return( true );
}


//==============================================================================
