//~~ trace.hh ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
/*
 *  description
 *
 * @author:    Andy Pippin   <pippin@cs.ucsb.edu>
 * @version:   $Date: 2004/05/17 03:07:43 $   $Revision: 1.2 $
 * @requires:   
 */

//==[ Identity ]================================================================

#ifndef  __TRACE_HH__
#define  __TRACE_HH__

#ifdef __TRACE_CC__
static char* hdrIdentity =
    "$Id: trace.hh,v 1.2 2004/05/17 03:07:43 pippin Exp $";
#endif


//==[ Includes ]================================================================

#include  <stdarg.h>


//==[ Typedefs/Structures/Enums ]===============================================

//==[ Constants ]===============================================================

//==[ Variables ]===============================================================

//==[ Classes ]=================================================================

//==[ Prototypes ]==============================================================


namespace  trace
{

    //----------------------------------------------------------------------
    /**
     *  Print something to standard out.
     *
     * @param   file  The name of the file (use the "__FILE__" macro).
     * @param   line  The line from the file (use the "__LINE__" macro).
     * @param   fmt   The printf-style format string.
     * @param   ...   Arguments.
     * @return  True.
     */
	    extern bool
    dprintf(
	char*  file,
	int    line,
	char*  fmt,
	...
    );


    //----------------------------------------------------------------------
    /**
     *  Print something to a "trace" log (trace.<rank#>).
     *
     * @param   file  The name of the file (use the "__FILE__" macro).
     * @param   line  The line from the file (use the "__LINE__" macro).
     * @param   fmt   The printf-style format string.
     * @param   ...   Arguments.
     * @return  True.
     */
	    extern bool
    log(
	char*  file,
	int    line,
	char*  fmt,
	...
    );


    //----------------------------------------------------------------------
    /**
     *  Clear the log.
     *
     * @return  True.
     */
	    extern bool
    clear(
	void
    );

};


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#endif // __TRACE_HH__
