//~~ usage.cc ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
/*
 *  description
 *
 * @author:    Andy Pippin   <pippin@cs.ucsb.edu>
 * @version:   $Date: 2004/05/30 00:58:21 $   $Revision: 1.2 $
 * @requires:   
 */


//==[ Identity ]================================================================

#define  __USAGE_CC__
static char* srcIdentity = 
    "$Id: usage.cc,v 1.2 2004/05/30 00:58:21 pippin Exp $";


//==[ Includes ]================================================================

#include  <string.h>

#include "john-mpi.hh"
#include "usage.hh"


//==[ Typedefs/Structures/Enums ]===============================================

//==[ Constants ]===============================================================

//==[ Variables ]===============================================================

//==[ Classes ]=================================================================

//==[ Prototypes ]==============================================================

//------------------------------------------------------------------------------
/*
 *  description
 *
 * @param   arg   description
 * @param   arg   description
 * @return  returns
 */


//==[ Implementation ]==========================================================

	static char
IsEqual(
    char*	str1,
    char*	str2
){
    char  result = 0;
    int   len1, len2;

    if( (NULL != str1) && (NULL != str2) )
    {
        len1 = strlen( str1 );
        len2 = strlen( str2 );

        result = !strncasecmp( str1, str2, ((len1 < len2) ? len1 : len2) );
    }

    return( result );
}


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	void 
Usage::print(
    char*	programName,
    char*	prefix,
    FILE*	outStream
){
    // Fix parameters.
    //
    programName = (NULL != programName) ? programName : john_mpi::cmdName;
    prefix = (NULL != prefix) ? prefix : (char*)"";
    outStream = (NULL != outStream) ? outStream : stdout;

    fprintf( outStream, "%sUsage: %s [-hqVv] [-i file] [-m #] [-o file] [-w file]\n", prefix, programName );
    fflush( outStream );

    return;
}


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

	void
Usage::help(
    char*	helpTopic,
    char*	programName,	
    char*	prefix,
    FILE*	outStream
){
    // Local variables.
    //
    bool  isNull;
    bool  isAll;

    // Set flags.
    //
    isNull = (NULL == helpTopic) ? true : false;
    if( ! isNull )
	isAll = (0 == strcasecmp( "all", helpTopic )) ? true : false;


    // Fix parameters.
    //
    programName = (NULL != programName) ? programName : john_mpi::cmdName;
    helpTopic = (NULL != helpTopic) ? helpTopic : (char*)"";
    prefix = (NULL != prefix) ? prefix : (char*)"";
    outStream = (NULL != outStream) ? outStream : stdout;


    // *** Now start displaying help. ***

    // Description
    //
    if( isNull || isAll || IsEqual( helpTopic, "desc" ) )
    {
        fprintf( outStream, "%s\n%s%s%s\n%s\n", prefix, prefix,
		    "--[ Description ]---------------------"
		    "--------------------------------------", prefix );

        fprintf( outStream,
		"%s  \"%s\" is an \"MPI-ized\" version of \"John the Ripper\".  John the\n%s  Ripper is Copyright (c) 1996-98 by Solar Designer\n",
	    prefix,
	    programName,
	    prefix );
        fprintf( outStream, "%s\n", prefix );
    }

    // Usage
    //
    if( isNull || isAll || IsEqual( helpTopic, "usage" ) )
    {
        fprintf( outStream, "%s\n%s%s%s\n%s\n", prefix, prefix,
		    "--[ Usage ]---------------------------"
		    "--------------------------------------", prefix );

        char  usagePrefixBuffer[ 80 ];
	snprintf( usagePrefixBuffer, sizeof(usagePrefixBuffer),
	    "%s  ", prefix );
	Usage::print( programName, usagePrefixBuffer, outStream );
        fprintf( outStream, "%s\n", prefix );
    }

    // Options
    //
    if( isNull || isAll || IsEqual( helpTopic, "option" ) )
    {
        fprintf( outStream, "%s\n%s%s%s\n%s\n", prefix, prefix,
		    "--[ Options ]-------------------------"
		    "--------------------------------------", prefix );

        fprintf( outStream,
"%s  If a long option shows an argument as mandatory, then it is mandatory\n\
%s  for the equivalent short option also.  Similarly for optional arguments.\n",
	    prefix,
	    prefix );
        fprintf( outStream, "%s\n", prefix );

	// Main operation.
	//
        fprintf( outStream, "%s\n%s%s%s\n%s\n", prefix, prefix,
		    "--[ Main Operation Modes ]------------"
		    "--------------------------------------", prefix );

        fprintf( outStream, 
"%s  -i, --password=file    Specify the password (input) file (def = \"%s\").\n\
%s  -m, --max=#            Maximum number of passwords per message (def = %d).\n\
%s  -o, --results=file     Specify the results (output) file (def = \"%s\").\n\
%s  -w, --wordlist=file    Specify the wordlist file (def = \"%s\").\n",
		prefix, john_mpi::DEFAULT_PasswordFilename,
		prefix, john_mpi::DEFAULT_MaxPasswords,
		prefix, john_mpi::DEFAULT_ResultsFilename,
		prefix, john_mpi::DEFAULT_WordlistFilename );
        fprintf( outStream, "%s\n", prefix );


	// Informative/Debug options.
	//
        fprintf( outStream, "%s\n%s%s%s\n%s\n", prefix, prefix,
		    "--[ Informative/Debug Modes ]---------"
		    "--------------------------------------", prefix );

        fprintf( outStream, 
"%s  -h, --help[=topic]     Display this help message or details on a topic.\n\
%s      --usage            Display synopsis of command line arguments.\n\
%s  -v, --verbose[=level]  Set verbosity (def = %d)\n\
%s  -q, --quiet            Set verbosity to 0.\n",
		prefix,
		prefix,
		prefix, john_mpi::DEFAULT_Verbosity,
		prefix,
		prefix );
        fprintf( outStream, "%s\n", prefix );
    }

    // Topics
    //
    if( isAll || IsEqual( helpTopic, "topic" ) )
    {
        fprintf( outStream, "%s\n%s%s%s\n%s\n", prefix, prefix,
		    "--[ Help Topics ]---------------------"
		    "--------------------------------------", prefix );

        fprintf( outStream, 
"%s  all       Display this help message or details on a topic.\n\
%s  desc      Description of this command.\n\
%s  option    Description of command line arguments.\n\
%s  topic     Help topics.\n\
%s  usage     Synopsis of command line arguments.\n\
%s\n",
		prefix,
		prefix,
		prefix,
		prefix,
		prefix,
		prefix );
    }

    fflush( outStream );
    return;
}



//==============================================================================
