//~~ worker.hh ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
/*
 *  description
 *
 * @author:    Andy Pippin   <pippin@cs.ucsb.edu>
 * @version:   $Date: 2004/05/16 23:04:04 $   $Revision: 1.2 $
 * @requires:   
 */

//==[ Identity ]================================================================

#ifndef  __WORKER_HH__
#define  __WORKER_HH__

#ifdef __WORKER_CC__
static char* hdrIdentity =
    "$Id: worker.hh,v 1.2 2004/05/16 23:04:04 pippin Exp $";
#endif


//==[ Includes ]================================================================

#include  <string>
#include  "workmsg.hh"

//==[ Typedefs/Structures/Enums ]===============================================

//==[ Constants ]===============================================================

//==[ Variables ]===============================================================

//==[ Classes ]=================================================================

class Worker
{
  public: //====================================================================

    //~ Member Typedefs/Structures/Enums ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    //~ Member Constants ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


    //----------------------------------------------------------------------
    /**
     *  Create an instance with no arguments.
     */
    Worker(
        void
    );


    //----------------------------------------------------------------------
    /**
     *  Default destructor.
     */
    	    virtual
    ~Worker(
        void
    );


    //~~ Getters ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    /**  Get the rank of this worker.  */
    virtual int  getMyRank( void );

    /**  Get the nuymber of workers in this communicator.  */
    virtual int  getNumWorkers( void );


    //~~ Setters ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


    //~~ Methods ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    //----------------------------------------------------------------------
    /**
     *  Begin execution.
     *
     * @return   Exit code (0 -=> no errors).
     */
    virtual int  start(
	void
    );


    //----------------------------------------------------------------------
    /**
     *  Attempt to break the password.
     *
     * @param   workMsg  Passwords to crack.
     * @return  Number of passwords cracked (negative number on error).
     */
            virtual int
    crackPassword(
	WorkMsg*  workMsg
    );


    //----------------------------------------------------------------------
    /**
     *  Make a string representation out of the instance.
     *
     * @param   prefix  Prepend this to each line (NULL --> "").
     * @param   label   Top-level label (NULL --> Worker).
     * @return  XML representation.
     */
            virtual char*
    toString(
	char*  prefix =NULL,
	char*  label  =NULL
    );


    //----------------------------------------------------------------------
    /**
     *  Test this class.
     *
     * @return   Success (true) or failure (false).
     */
    static bool  test( void );



  protected: //===============================================================

    int  myRank;
    int  numWorkers;


  private: //=================================================================

    //~~ Member variables ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


    //----------------------------------------------------------------------
    /**
     *  Initialize the member variables.
     */
    	    virtual void
    Initialize(
        void
    );

};


//==[ Prototypes ]==============================================================

//------------------------------------------------------------------------------
/**
 *  description
 *
 * @param   arg   description
 * @param   arg   description
 * @return  returns
 */


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#endif // __WORKER_HH__
