//~~ workmsg.cc ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
/*
 *  description
 *
 * @author:    Andy Pippin   <pippin@cs.ucsb.edu>
 * @version:   $Date: 2004/05/31 03:47:27 $   $Revision: 1.12 $
 * @requires:   
 */


//==[ Identity ]================================================================

#define  __WORKMSG_CC__
static char* srcIdentity = 
    "$Id: workmsg.cc,v 1.12 2004/05/31 03:47:27 pippin Exp $";


//==[ Includes ]================================================================

#include  <errno.h>
#include  <unistd.h>
#include  <string>

#include  "john-mpi.hh"
#include  "trace.hh"
#include  "workmsg.hh"


//==[ Typedefs/Structures/Enums ]===============================================

//==[ Constants ]===============================================================

static int const  SHUTDOWN_TAG  = 75647; // You figure it out.... :)

static int const  NUM_PASSWORDS   = 0;
static int const  SEQUENCE_ID     = 1;
static int const  CRACKS_PER_SEC  = 2;

static int const  KEY_BUFFER_LEN  = 6;


//==[ Variables ]===============================================================

static int  sequenceNumber = 0;


//==[ Classes ]=================================================================

//==[ Prototypes ]==============================================================

//------------------------------------------------------------------------------
/*
 *  description
 *
 * @param   arg   description
 * @param   arg   description
 * @return  returns
 */


//==[ Implementation ]==========================================================

	void
WorkMsg::Initialize(
    void
){
    this->sourceRank   = MPI::ANY_SOURCE;
    this->sourceTag    = MPI::ANY_TAG;
    this->numPasswords = 0;
    this->sequenceId   = sequenceNumber++;

    for( register int p=0; p < WorkMsg::MAX_PASSWORDS; ++p )
    {
        this->encrypted[ p ] = NULL;
        this->decrypted[ p ] = NULL;
    }

    return;
}


//------------------------------------------------------------------------------

WorkMsg::WorkMsg(
    void
){
    Initialize();
    return;
}


//------------------------------------------------------------------------------

WorkMsg::WorkMsg(
    int    numHashes,
    char*  hash[]
){
    Initialize();

    if( numHashes < WorkMsg::MAX_PASSWORDS )
    {
	this->sourceRank = MPI::COMM_WORLD.Get_rank();
	this->numPasswords = numHashes;

	for( int p=0; p < this->numPasswords; ++p )
	{
	    if( NULL != hash[ p ] )
		this->encrypted[p] = strdup( hash[p] );
	}
    }

    return;
}


//------------------------------------------------------------------------------

WorkMsg::~WorkMsg(
    void
){
    for( register int p=0; p < WorkMsg::MAX_PASSWORDS; ++p )
    {
        if( NULL != this->encrypted[ p ] )
	    free( this->encrypted[ p ] );

        if( NULL != this->decrypted[ p ] )
	    free( this->decrypted[ p ] );
    }

    return;
}


//------------------------------------------------------------------------------

	WorkMsg*
WorkMsg::Recv(
    int  source,
    int  tag
){
    WorkMsg*  recvMsg = new WorkMsg();
    MPI::Status  mpiStatus;
    char  enBuffer[ WorkMsg::MAX_ENCRYPTED_LENGTH ];
    char  deBuffer[ WorkMsg::MAX_DECRYPTED_LENGTH ];
    int   keyBuffer[ KEY_BUFFER_LEN ];


    // And here they come...
    //
    if( NULL != recvMsg )
    {
	// Receive the number of passwords.
	//
	MPI::COMM_WORLD.Recv( keyBuffer, KEY_BUFFER_LEN,
				    MPI::INT, source, tag, mpiStatus );

	recvMsg->numPasswords     = keyBuffer[ NUM_PASSWORDS  ];
	recvMsg->sequenceId       = keyBuffer[ SEQUENCE_ID    ];
	recvMsg->cracksPerSecond  = keyBuffer[ CRACKS_PER_SEC ];

	recvMsg->compute[ TIME_Hours    ] = keyBuffer[ 3+TIME_Hours    ];
	recvMsg->compute[ TIME_Minutes  ] = keyBuffer[ 3+TIME_Minutes  ];
	recvMsg->compute[ TIME_Seconds  ] = keyBuffer[ 3+TIME_Seconds  ];

	recvMsg->sourceRank = mpiStatus.Get_source();
	recvMsg->sourceTag  = mpiStatus.Get_tag();


	// If this isn't a shutdown message, the hoover in the passwords.
	//
	if( !recvMsg->Shutdown()
	 && (0 < recvMsg->numPasswords) )
	{
	    for( int p=0; p < recvMsg->numPasswords; ++p )
	    {
	        memset( enBuffer, 0, WorkMsg::MAX_ENCRYPTED_LENGTH );
		MPI::COMM_WORLD.Recv(
			enBuffer, WorkMsg::MAX_ENCRYPTED_LENGTH, MPI::CHAR,
			recvMsg->sourceRank, recvMsg->sourceTag );
		recvMsg->setEncrypted( p, enBuffer );

	        memset( deBuffer, 0, WorkMsg::MAX_DECRYPTED_LENGTH );
		MPI::COMM_WORLD.Recv(
			deBuffer, WorkMsg::MAX_DECRYPTED_LENGTH, MPI::CHAR,
			recvMsg->sourceRank, recvMsg->sourceTag );
		recvMsg->setDecrypted( p, deBuffer );
	    }

	} // End no shutdown.
    }

    return( recvMsg );
}


//------------------------------------------------------------------------------

	void
WorkMsg::SendShutdown(
    int  dest
){
    int  keyBuffer[ 5 ];

    if( 0 == MPI::COMM_WORLD.Get_rank() )
    {
	keyBuffer[ NUM_PASSWORDS   ] = 0;
	keyBuffer[ SEQUENCE_ID     ] = ++sequenceNumber;

        MPI::COMM_WORLD.Send( keyBuffer, KEY_BUFFER_LEN,
				    MPI::INT,  dest, SHUTDOWN_TAG );
    }

    return;
}


//------------------------------------------------------------------------------

	bool
WorkMsg::Send(
    int  dest,
    int  tag
){
    bool  success = true;
    char  enBuffer[ WorkMsg::MAX_ENCRYPTED_LENGTH ];
    char  deBuffer[ WorkMsg::MAX_DECRYPTED_LENGTH ];
    int   keyBuffer[ KEY_BUFFER_LEN ];


    if( SHUTDOWN_TAG == tag )
    {
        return( false );
	//    *************************
	//   ***  DOES NOT CONTINUE  ***
	//    *************************
    }

    // And away they go...
    //
    keyBuffer[ NUM_PASSWORDS   ] = this->numPasswords;
    keyBuffer[ SEQUENCE_ID     ] = this->sequenceId;
    keyBuffer[ CRACKS_PER_SEC  ] = this->cracksPerSecond;
    keyBuffer[ 3+TIME_Hours    ] = this->compute[ TIME_Hours    ];
    keyBuffer[ 3+TIME_Minutes  ] = this->compute[ TIME_Minutes  ];
    keyBuffer[ 3+TIME_Seconds  ] = this->compute[ TIME_Seconds  ];

    MPI::COMM_WORLD.Send( keyBuffer, KEY_BUFFER_LEN, MPI::INT, dest, tag );

    for( int p=0; p < this->numPasswords; ++p )
    {
	// Send the encrypted password.
	//
	memset( enBuffer, 0, WorkMsg::MAX_ENCRYPTED_LENGTH );
        strncpy( enBuffer, this->encrypted[p], sizeof(enBuffer) );
	enBuffer[ WorkMsg::MAX_ENCRYPTED_LENGTH-1 ] = '\0';
	MPI::COMM_WORLD.Send( enBuffer, WorkMsg::MAX_ENCRYPTED_LENGTH,
				MPI::CHAR, dest, tag );

	// Send the decrypted password.
	//
	memset( deBuffer, 0, WorkMsg::MAX_DECRYPTED_LENGTH );
	if( NULL != this->decrypted[p] )
	{
	    strncpy( deBuffer, this->decrypted[p], sizeof(deBuffer)-1 );
	}

	MPI::COMM_WORLD.Send( deBuffer, WorkMsg::MAX_ENCRYPTED_LENGTH,
				MPI::CHAR, dest, tag );
    }

    return( success );
}


//-- Getters -------------------------------------------------------------------

int  WorkMsg::getSource( void )
    { return( this->sourceRank ); }

int  WorkMsg::getTag( void )
    { return( this->sourceTag ); }

int  WorkMsg::getSequence( void )
    { return( this->sequenceId ); }

int  WorkMsg::getNumEntries( void )
    { return( numPasswords ); }

int*  WorkMsg::getCompute( void )
    { return( compute ); }

bool  WorkMsg::Shutdown( void )
    { return( (SHUTDOWN_TAG == this->sourceTag) ? true : false ); }

int  WorkMsg::getCracksPerSecond( void )
    { return( cracksPerSecond ); }

char*  WorkMsg::getEncrypted( int index )
{
    char* result = NULL;

    if( (0 <= index) && (WorkMsg::MAX_PASSWORDS > index) )
         result = this->encrypted[ index ];

    return( result );
}

char*  WorkMsg::getDecrypted( int index )
{
    char* result = NULL;

    if( (0 <= index) && (WorkMsg::MAX_PASSWORDS > index) )
         result = this->decrypted[ index ];

    return( result );
}



void  WorkMsg::setCracksPerSecond( int cracksPerSecond )
    { this->cracksPerSecond = cracksPerSecond;  return; }


//------------------------------------------------------------------------------

	void
WorkMsg::setEncrypted(
    int index,
    char* encrypted
){
    if( (0 <= index)
     && (WorkMsg::MAX_PASSWORDS > index)
     && (NULL != encrypted) )
    {
	this->encrypted[ index ] = strdup( encrypted );
    }

    return;
}


//------------------------------------------------------------------------------

	void
WorkMsg::setDecrypted(
    int index,
    char* decrypted
){

    if( (0 <= index)
     && (WorkMsg::MAX_PASSWORDS > index)
     && (NULL != decrypted)
     && (0 < strlen( decrypted )) )
    {
	this->decrypted[ index ] = strdup( decrypted );
    }
}


//------------------------------------------------------------------------------

	void
WorkMsg::setDecrypted(
    char* encrypted,
    char* decrypted
){
    for( register int entry=0; entry < numPasswords; ++entry )
    {
        if( 0 == strcmp( encrypted, this->encrypted[ entry ] ) )
	{
	    this->decrypted[ entry ] = strdup( decrypted );
	    break;
	}
    }

    return;
}


//------------------------------------------------------------------------------

	void
WorkMsg::setCompute(
    int  hours,
    int  minutes,
    int  seconds
){
    this->compute[ TIME_Hours    ] = hours;
    this->compute[ TIME_Minutes  ] = minutes;
    this->compute[ TIME_Seconds  ] = seconds;

    return;
}


//------------------------------------------------------------------------------

	void
WorkMsg::setCompute(
    int p_compute[3]
){
    this->compute[ TIME_Hours    ] = p_compute[ TIME_Hours    ];
    this->compute[ TIME_Minutes  ] = p_compute[ TIME_Minutes  ];
    this->compute[ TIME_Seconds  ] = p_compute[ TIME_Seconds  ];

    return;
}


//------------------------------------------------------------------------------

	char*
WorkMsg::toString(
    char*  prefix,
    char*  label
){
    static std::string  result;
    static char  line[ 256 ];


    // Initialize and fix NULLs.
    //
    result.erase();
    if( NULL == prefix )  prefix = "";
    if( NULL == label  )  label  = "WorkMsg";

    snprintf( line, sizeof(line), "%s<%s sequenceId=\"%d\">\n",
				    prefix, label, this->sequenceId );
    result += line;

    snprintf( line, sizeof(line), "%s  <ComputeTime>\n", prefix );
    result += line;

    snprintf( line, sizeof(line), "%s    <Hours>%d</Hours>\n",
			    prefix, this->compute[ TIME_Hours   ] );
    result += line;

    snprintf( line, sizeof(line), "%s    <Minutes>%d</Minutes>\n",
			    prefix, this->compute[ TIME_Minutes ] );
    result += line;

    snprintf( line, sizeof(line), "%s    <Seconds>%d</Seconds>\n",
			    prefix, this->compute[ TIME_Seconds ] );
    result += line;

    snprintf( line, sizeof(line), "%s  </ComputeTime>\n", prefix );
    result += line;

    if( 0 == numPasswords )
	snprintf( line, sizeof(line), "%s  <Passwords size=\"0\"/>\n", prefix );
    else
	snprintf( line, sizeof(line), "%s  <Passwords size=\"%d\">\n",
		    prefix, numPasswords );
    result += line;

    for( int p=0; p < numPasswords; ++p )
    {
	snprintf( line, sizeof(line), "%s    <Password>\n", prefix );
	result += line;

	snprintf( line, sizeof(line), "%s      <Encrypted>%s</Encrypted>\n",
			prefix, encrypted[p] );
	result += line;

	if( NULL != decrypted[p] )
	    snprintf( line, sizeof(line), "%s      <Decrypted>%s</Decrypted>\n",
			    prefix, decrypted[p] );
	else
	    snprintf( line, sizeof(line), "%s      <Decrypted/>\n", prefix );
	result += line;

	snprintf( line, sizeof(line), "%s    </Password>\n", prefix );
	result += line;
    }

    if( 0 != numPasswords )
    {
	snprintf( line, sizeof(line), "%s  </Passwords>\n", prefix );
	result += line;
    }

    snprintf( line, sizeof(line), "%s  <CracksPerSecond>%d</CracksPerSecond>\n",
			    prefix, this->cracksPerSecond );
    result += line;

    snprintf( line, sizeof(line), "%s</%s>\n", prefix, label );
    result += line;

    return( (char*)result.c_str() );
}


//------------------------------------------------------------------------------

	bool
WorkMsg::CreatePasswd(
    char* passwdFilename
){
    bool  success =false;
    FILE*  passwd;

    if( NULL != passwdFilename )
    {
        if( NULL == (passwd = fopen( passwdFilename, "w" )) )
	{
	    char  errorBuffer[ 128 ];
	    strncpy( errorBuffer, strerror( errno ), sizeof(errorBuffer) );
	    trace::dprintf( __FILE__, __LINE__,
		"ERROR  Couldn't open \"%s\": %s\n",
		passwdFilename, errorBuffer );
	}
	else
	{
	    for( int p=0; p < this->numPasswords; ++p )
	    {
	        fprintf( passwd, "%02d:%s:65535:65535:gecos:homedir:shell\n",
					p, encrypted[ p ] );
	    }

	    fclose( passwd );
	    success = true;
	}
    }
    else
    {
        trace::dprintf( __FILE__, __LINE__, "%s%s\n",
			    "ERROR  WorkMsg::CreatePassword() - ",
			    "Received NULL passwdFilename!\n" );
    }

    return( success );
}

//==============================================================================
