//~~ workmsg.h ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
/*
 *  description
 *
 * @author:    Andy Pippin   <pippin@cs.ucsb.edu>
 * @version:   $Date: 2004/05/31 03:47:27 $   $Revision: 1.11 $
 * @requires:   
 */

//==[ Identity ]================================================================

#ifndef  __WORKMSG_H__
#define  __WORKMSG_H__

#ifdef __WORKMSG_CC__
static char* hdrIdentity =
    "$Id: workmsg.hh,v 1.11 2004/05/31 03:47:27 pippin Exp $";
#endif


//==[ Includes ]================================================================

#include  <stdarg.h>
#include  <string>
#include  "mpi.h"


//==[ Typedefs/Structures/Enums ]===============================================

//==[ Constants ]===============================================================

//==[ Variables ]===============================================================

//==[ Classes ]=================================================================

class WorkMsg
{
  public: //====================================================================

    //~ Member Typedefs/Structures/Enums ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    //~ Member Constants ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    static int const  MAX_PASSWORDS        = 20;
    static int const  MAX_ENCRYPTED_LENGTH = 32;
    static int const  MAX_DECRYPTED_LENGTH = 32;

    static int const  TIME_Hours           = 0;
    static int const  TIME_Minutes         = 1;
    static int const  TIME_Seconds         = 2;


    //----------------------------------------------------------------------
    /**
     *  Create an instance with no arguments.
     */
    WorkMsg(
        void
    );


    //----------------------------------------------------------------------
    /**
     *  Create an instance with the specified strings.
     */
    WorkMsg(
        int    numPasswords,
	char*  encrypted[]
    );


    //----------------------------------------------------------------------
    /**
     *  Default destructor.
     */
    	    virtual
    ~WorkMsg(
        void
    );


    //----------------------------------------------------------------------
    /**
     *  Receive a message.
     *
     * @param   source  Source rank process.
     * @param   tag     Associated tag (avoid using big numbers).
     * @result  The received WorkMsg.
     */
	    static WorkMsg*
    Recv(
	int  source =MPI::ANY_SOURCE,
	int  tag    =MPI::ANY_TAG
    );



    //~~ Send/Receive ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    //----------------------------------------------------------------------
    /**
     *  Send this instance to the specified process rank.
     *
     * @param   dest   Target rank process.
     * @param   tag    Associated tag (avoid using big numbers).
     * @return  Success or failure.
     */
	    virtual bool
    Send(
	int dest,
	int tag  =0
    );


    //----------------------------------------------------------------------
    /**
     *  Send the specified destination a non-blocking shutdown message (an
     *  empty message w/ a special tag.  Note that the Send message above
     *  cannot send a message with the "shutdown tag", so it should avoid
     *  sending messages with large values for the tag.
     *
     * @param   dest   Target rank process.
     */
	    static void
    SendShutdown(
	int dest
    );


    //~~ Getters ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    /**  Who sent the message.  */
    virtual int  getSource( void );

    /**  How was the message tagged.  */
    virtual int  getTag( void );

    /**  What is the sequence number?  */
    virtual int  getSequence( void );

    /**  Get the numEntries member variable of this class.  */
    virtual int  getNumEntries( void );

    /**  Get a particular encrypted password of this class.  */
    virtual char*  getEncrypted( int index );

    /**  Get a particular decrypted password of this class.  */
    virtual char*  getDecrypted( int index );

    /**  Is the shutdown tag set?  */
    virtual bool  Shutdown( void );

    /**  Number of hours, minutes, seconds and microseconds of compute time.  */
    virtual int*  getCompute( void );

    /**  Get the cracks per second.  */
    virtual int  getCracksPerSecond( void );


    //----------------------------------------------------------------------
    /**
     *  Make a string representation out of the instance.
     *
     * @param   prefix  Preceed each line with this.
     * @param   label   Top-level label.
     * @return  String representation.
     */
            virtual char*
    toString(
	char*  prefix =NULL,
	char*  label  =NULL
    );


    //~~ Setters ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    /**  Set a particular encrypted password of this class.  */
    virtual void  setEncrypted( int index, char* encrypted );

    /**  Set a particular decrypted password of this class.  */
    virtual void  setDecrypted( int index, char* decrypted );

    /**  Set a particular decrypted password.  */
    virtual void  setDecrypted( char* encrypted, char* decrypted );

    /**  Set the compute time.  */
    virtual void  setCompute( int hr, int min, int sec );

    /**  Set the compute time.  */
    virtual void  setCompute( int computeTime[3] );

    /**  Set the cracks per second.  */
    virtual void  setCracksPerSecond( int cracksPerSecond );


    //----------------------------------------------------------------------
    /**
     *  Create a password file.
     *
     * @return   Success (true) or failure (false).
     */
    virtual bool  CreatePasswd( char* passwdFilename );


  protected: //===============================================================

  private: //=================================================================

    //~~ Member variables ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    int    sourceRank;
    int    sourceTag;
    int    sequenceId;
    int    numPasswords;
    int    compute[4];
    int    cracksPerSecond;
    char*  encrypted[ WorkMsg::MAX_PASSWORDS ];
    char*  decrypted[ WorkMsg::MAX_PASSWORDS ];


    //----------------------------------------------------------------------
    /**
     *  Initialize the member variables.
     */
    	    virtual void
    Initialize(
        void
    );

};


//==[ Prototypes ]==============================================================

//------------------------------------------------------------------------------
/**
 *  description
 *
 * @param   arg   description
 * @param   arg   description
 * @return  returns
 */


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#endif // __WORKMSG_H__
