//~~ crack.cc ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
/*
 *  There are already too many files beginning with "john" and "work", so
 *  I called this "crack.cc"
 *
 * @author:    Andy Pippin   <pippin@cs.ucsb.edu>
 * @version:   $Date: 2004/06/03 03:09:47 $   $Revision: 1.10 $
 * @requires:   
 */


//==[ Identity ]================================================================

#define  __CRACK_CC__
static char* srcIdentity = 
    "$Id: crack.cc,v 1.10 2004/06/03 03:09:47 pippin Exp $";


//==[ Includes ]================================================================

#include  <errno.h>
#include  <fcntl.h>
#include  <string.h>
#include  <sys/time.h>
#include  <sys/types.h>
#include  <unistd.h>
#include  "mpi.h"

#include  "john-mpi.hh"
#include  "crack.hh"
#include  "trace.hh"
#include  "workmsg.hh"
#include  "john-1.6/src/worker.h"


//==[ Typedefs/Structures/Enums ]===============================================

//==[ Constants ]===============================================================

static       int   const  JOHN_MaxArguments = 10;
static const char* const  JOHN_StderrFilename = "john-stderr";


//==[ Variables ]===============================================================

//==[ Classes ]=================================================================

//==[ Prototypes ]==============================================================

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
/**
 *  Convert the results to a string.
 *
 * @param   results  Worker results.
 * @return  A "Java" string.
 */
	static char*
results_toString(
    struct worker_result*  results
);


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
/**
 *  Process the results from John.
 *
 * @param   workMsg  The work message from the dealer.
 * @param   results  The results from John.
 * @return  Number of passwords cracked.
 */
	static int
ProcessResults(
    WorkMsg*  workMsg,
    struct worker_result*  results
);


//==[ Implementation ]==========================================================

	void
Crack::Initialize(
    void
){
    this->wordlistFilename = NULL;

    snprintf( john_mpi::passwdFile, FILENAME_MAX,
					"/tmp/john-passwd.%d", getpid() );

    // Throw standard output into the trash.
    //
    close( 1 );
    open( "/dev/null", O_WRONLY );

    // Grab hold of standard error.
    //
    close( 2 );
    this->stdError = open( JOHN_StderrFilename, O_WRONLY|O_CREAT|O_TRUNC|O_SYNC,
				    S_IRUSR|S_IWUSR );

    if( 2 != this->stdError )
        trace::dprintf( __FILE__, __LINE__, "Wasn't able to grab stderr!\n" );

    return;
}


//------------------------------------------------------------------------------

Crack::Crack(
    void
){
    Initialize();
    return;
}


//------------------------------------------------------------------------------

Crack::Crack(
    char*  wordlistFilename
){
    Initialize();
    setWordlistFilename( wordlistFilename );
    return;
}


//------------------------------------------------------------------------------

Crack::~Crack(
    void
){
    if( NULL != this->wordlistFilename )
        free( this->wordlistFilename );

    // Close and remove standard out.
    //
    if( 1 == this->stdError )
    {
        close( this->stdError );
	unlink( john_mpi::passwdFile );
	unlink( JOHN_StderrFilename );

	worker_DeletePOTFile();
    }
}


//------------------------------------------------------------------------------

	int
Crack::start(
    void
){
    int        exitCode = 0;			// Exit code.
    WorkMsg*   workMsg  = new WorkMsg();	// Initial message.
    WorkMsg*   recvMsg;				// New work message.
    bool       shutdown = false;		// Shutdown?
    time_t     rightNow;			// The time right.... NOW!
    struct tm  startTime;			// Start time.
    struct tm  stopTime;			// Stop time.


    // Initialize.
    trace::clear();


    // Keep looping while everything is kosher and we're not told to shutdown.
    //
    while( (0 <= exitCode) && !shutdown )
    {
        // Send the results of the previous crack.
	//
	if( false == workMsg->Send( 0 ) )
	{
	    exitCode = -1;
	    if( 0 < john_mpi::verboseLevel )
	    {
		trace::dprintf( __FILE__, __LINE__,
		    "ERROR  Couldn't send message from %d to 0!\n", myRank );
	    }
	}

	// Make sure we receive a message.
	//
	else if( NULL == (recvMsg = WorkMsg::Recv( 0 )) )
	{
	    exitCode = -2;
	    if( 0 < john_mpi::verboseLevel )
	    {
		trace::dprintf( __FILE__, __LINE__,
		    "ERROR  Recieved NULL message from %d!\n", 0 );
	    }
	}

	// Process the message.
	//
	else
	{
	    // Delete the previous message.
	    //
	    delete  workMsg;
	    workMsg = recvMsg;

	    // Shutdown message?  If not, work on this password.
	    //
	    if( workMsg->Shutdown() )
	    {
		shutdown = true;
	    }
	    else
	    {
		rightNow = time(0);
		(void)localtime_r( &rightNow, &startTime );

		exitCode = crackPassword( workMsg );

		rightNow = time(0);
		(void)localtime_r( &rightNow, &stopTime );

		// Compute the time.
		//
		int  hours   = stopTime.tm_hour - startTime.tm_hour;
		int  minutes = stopTime.tm_min  - startTime.tm_min;
		int  seconds = stopTime.tm_sec  - startTime.tm_sec;

		if( 0 > seconds )
		{
		    seconds += 60;
		    minutes -= 1;
		}
		if( 0 > minutes )
		{
		    minutes += 60;
		    hours   -= 1;
		}

		// Set the time.
		//
		workMsg->setCompute( hours, minutes, seconds );
	    }
        }
    }

    return( exitCode );
}


//------------------------------------------------------------------------------

	int
Crack::crackPassword(
    WorkMsg*  workMsg
){
    int    numCracked = 0;
    char*  argv[ JOHN_MaxArguments ];
    int    argc = 0;
    char   wordlistBuffer[ 80 ];
    struct worker_result*  results;
    off_t  stderrPosition, bytesRead;
    char   readBuffer[ 256 ];
    FILE*  Stderr = NULL;		// Can't read on file descriptor 2!
    char*  cracksPerSecond;
    char*  space;


    if( (NULL != this->wordlistFilename )
     && workMsg->CreatePasswd( john_mpi::passwdFile ) )
    {
	// Build the appropriate argc/argv.
	//
	argv[ argc++ ] = "john";
	snprintf( wordlistBuffer, sizeof(wordlistBuffer),
					"-w=%s", this->wordlistFilename );
	argv[ argc++ ] = wordlistBuffer;
	argv[ argc++ ] = "-rules";
	argv[ argc++ ] = john_mpi::passwdFile;
	argv[ argc ] = NULL;

	// Where are we in the standard error?
	//
	if( 2 == this->stdError )
	    stderrPosition = lseek( this->stdError, 0, SEEK_END );

	// Call John.
	//
	(void)worker_main( argc, argv );
	results = worker_getResults();

	// Where are we in the standard error now?
	//
	if( (2 == this->stdError)
	 && (NULL != (Stderr = fopen( JOHN_StderrFilename, "r" )) ) )
	{
	    if( (-1 != fseek( Stderr, stderrPosition, SEEK_SET ))
	     && (NULL != fgets( readBuffer, sizeof(readBuffer), Stderr )) )
	    {
		if( NULL != (cracksPerSecond = strstr( readBuffer, "c/s: " ))
		 && (NULL != (space = strchr( cracksPerSecond+5, ' ' ))) )
		{
		    *space = '\0';
		    results->crack_stat = atoi( cracksPerSecond+5 );
		}
	    }
	    fclose( Stderr );
	}

	// Process the results.
	//
	numCracked = ProcessResults( workMsg, results );
    }

    return( numCracked );
}


//-- Getters -------------------------------------------------------------------

char*  Crack::getWordlistFilename( void )
    { return( this->wordlistFilename ); }


//-- Setters -------------------------------------------------------------------

void  Crack::setWordlistFilename( char* wordlistFilename )
    { this->wordlistFilename = strdup( wordlistFilename );  return; }


//------------------------------------------------------------------------------

	char*
Crack::toString(
    char*  prefix,
    char*  label
){
    static std::string  result;
    static char  line[ 256 ];

    // Initialize and fix NULLs.
    //
    result.erase();
    if( NULL == prefix )  prefix = "";
    if( NULL == label  )  label  = "Worker";

    snprintf( line, sizeof(line), "%s<%s>\n", prefix, label );
    result += line;


    snprintf( line, sizeof(line), "%s</%s>\n", prefix, label );
    result += line;

    return( (char*)result.c_str() );
}


//------------------------------------------------------------------------------

	static int
ProcessResults(
    WorkMsg*  workMsg,
    struct worker_result*  results
){
    int    numberResults = 0;		// How many cracked?

    if( (NULL != workMsg)
     && (NULL != results) )
    {
        numberResults = results->num_elements;

	for( int r=0; r < numberResults; ++r )
	{
	    workMsg->setDecrypted( results->cracked[ r ][ ENCRYPTED ],
				   results->cracked[ r ][ DECRYPTED ] );
	}

	workMsg->setCracksPerSecond( results->crack_stat );
    }

    return( numberResults );
}


//------------------------------------------------------------------------------

	static char*
results_toString(
    struct worker_result*  results
){
    static char  buffer[ 1024 ];
    char  b[ 256 ];

    if( NULL != results )
    {
	strncpy( buffer, "<Results>\n", sizeof(buffer) );
	snprintf( b, 256, "  <num_elements>%d</num_elements>\n",
						results->num_elements );
	strncat( buffer, b, sizeof(buffer) );

	for( int p=0; p < results->num_elements; ++p )
	{
	    strncat( buffer, "  <cracked>\n", sizeof(buffer) );

	    snprintf( b, 256, "    <encrypted>%s</encrypted>\n",
					    results->cracked[p][ENCRYPTED] );
	    strncat( buffer, b, sizeof(buffer) );
	    snprintf( b, 256, "    <decrypted>%s</decrypted>\n",
					    results->cracked[p][DECRYPTED] );
	    strncat( buffer, b, sizeof(buffer) );

	    strncat( buffer, "  </cracked>\n", sizeof(buffer) );
	}

	snprintf( b, 256, "  <crack_stat>%d</crack_stat>\n",
						results->crack_stat );
	strncat( buffer, b, sizeof(buffer) );

	strncat( buffer, "</Results>\n", sizeof(buffer) );
    }
    else
    {
	strncpy( buffer, "<Results null=\"true\"/>\n", sizeof(buffer) );
    }

    return( buffer );
}


//==============================================================================
